/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  ** This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether 
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * COPYRIGHT(c) 2018 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f1xx_hal.h"

/* USER CODE BEGIN Includes */
#include "stdint.h"
#include "stdlib.h"
/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/
I2C_HandleTypeDef hi2c2;

UART_HandleTypeDef huart1;
DMA_HandleTypeDef hdma_usart1_rx;
DMA_HandleTypeDef hdma_usart1_tx;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/

//interupt variables
volatile uint8_t timer_display=0;	//0,01s
volatile uint8_t timer_switch=0;	//4s
volatile uint8_t timer_sensor=0;	//100s
volatile uint8_t timer_bluetooth;
volatile uint8_t bluetooth_timeout=0;

//disply variable

uint8_t Received[1];		//zmienna danych odbieranych przez bluetooth
uint8_t Data[140];			//zmienna do wysyania cigw przez bluetooth

uint8_t Settings=0;
int32_t t_fine;
uint16_t calibration_dig_T1;
int16_t calibration_dig_T2;
int16_t calibration_dig_T3;

uint16_t calibration_dig_P1;
int16_t calibration_dig_P2;
int16_t calibration_dig_P3;
int16_t calibration_dig_P4;
int16_t calibration_dig_P5;
int16_t calibration_dig_P6;
int16_t calibration_dig_P7;
int16_t calibration_dig_P8;
int16_t calibration_dig_P9;

uint8_t calibration_dig_H1;
int16_t calibration_dig_H2;
uint8_t calibration_dig_H3;
int16_t calibration_dig_H4;
int16_t calibration_dig_H5;
uint8_t calibration_dig_H6;

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_DMA_Init(void);
static void MX_I2C2_Init(void);
static void MX_USART1_UART_Init(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

//void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim);	//Timers configuration

//bme280 bosh sensor functions - i2c
uint8_t readRegister(uint16_t adress);
uint8_t writeRegister(uint16_t adress, uint8_t *pData);
uint8_t bme280_callibration();
float bme280_getTemperature();
uint16_t bme280_getHumidity();
uint16_t bme280_getPressure();
uint8_t bme280_state(uint8_t state);

//display functions 8segment led display - spi
void spiWrite(uint8_t adress, uint8_t data);
uint8_t cto8seg(uint8_t c);
uint8_t ito8seg(uint8_t a);
void display4x8seg(uint8_t a, uint8_t b, uint8_t c,uint8_t d);
void displaychar(char *b);
void displayint(int b);
//bluetooth interrupt
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart);


/* USER CODE END PFP */

/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

int main(void)
{

  /* USER CODE BEGIN 1 */
	uint8_t a=0;

	int16_t temperature;
	uint16_t humidity;
	uint16_t pressure;
	uint16_t error=0;

	uint8_t display_datasize=5;
	char display_data[display_datasize];

	uint8_t name_sensor=0;
	uint8_t name_sensor_pin1=0;
	uint8_t name_sensor_pin2=0;

  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_I2C2_Init();
  MX_USART1_UART_Init();

  /* USER CODE BEGIN 2 */
  	  //Configure BT name and parameter:


	//Init and callibration bme280 sensor
	bme280_callibration();

	//first measure
	temperature = (int16_t) bme280_getTemperature();
	humidity =  bme280_getHumidity();
	pressure =  bme280_getPressure();

	//Timers start


	//name sensor
	if (HAL_GPIO_ReadPin(name1_GPIO_Port, name1_Pin) == GPIO_PIN_SET){
	  name_sensor_pin1=1;
	}
	else{
	  name_sensor_pin1=0;
	}
	if (HAL_GPIO_ReadPin(name2_GPIO_Port, name2_Pin) == GPIO_PIN_SET){
	  name_sensor_pin2=1;
	}
	else{
	  name_sensor_pin2=0;
	}

	name_sensor=name_sensor_pin2+(name_sensor_pin1*2);


	//show hello communicate
	//snprintf(Data, 140,"\n========================\n Sensor nr%d IoT  \n Tomasz Pacholek \n \n AT+NAME=STACJAMET\r\n\n",name_sensor);
	snprintf(Data, 140,"AT+NAMESTACJA-METEO-%d\r\n\n",name_sensor);
	HAL_UART_Transmit_DMA(&huart1, Data, 140);
	HAL_UART_Receive_DMA(&huart1, Received, 1);

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
	  //Change display data - humidity, pressure, temperature or other informations
	  if(timer_bluetooth>0){

		  	bme280_state(1);

			temperature = (int16_t) bme280_getTemperature();
			humidity =  bme280_getHumidity();
			pressure =  bme280_getPressure();
			bme280_state(0);

			snprintf(Data, 140,"\n========================\nSensor nr %d \nTemperatura:\t%d\tC \nCisnienie:\t\t %d\thPa \nWilgotnosc:\t\t%d\t%% \n========================\n",name_sensor,((int16_t)temperature),((uint16_t)pressure),((uint16_t)humidity));

			HAL_UART_Transmit_DMA(&huart1, Data, 140);
			HAL_UART_Receive_DMA(&huart1, Received, 1); // start receive
		  timer_bluetooth=0;
		  HAL_PWR_EnterSLEEPMode(1, PWR_SLEEPENTRY_WFI);

	  }


  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */

  }
  /* USER CODE END 3 */

}

/** System Clock Configuration
*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 16;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV16;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Configure the Systick interrupt time 
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick 
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* I2C2 init function */
static void MX_I2C2_Init(void)
{

  hi2c2.Instance = I2C2;
  hi2c2.Init.ClockSpeed = 100000;
  hi2c2.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c2.Init.OwnAddress1 = 0;
  hi2c2.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c2.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c2.Init.OwnAddress2 = 0;
  hi2c2.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c2.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c2) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}

/* USART1 init function */
static void MX_USART1_UART_Init(void)
{

  huart1.Instance = USART1;
  huart1.Init.BaudRate = 9600;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}

/** 
  * Enable DMA controller clock
  */
static void MX_DMA_Init(void) 
{
  /* DMA controller clock enable */
  __HAL_RCC_DMA1_CLK_ENABLE();

  /* DMA interrupt init */
  /* DMA1_Channel4_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel4_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(DMA1_Channel4_IRQn);
  /* DMA1_Channel5_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel5_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(DMA1_Channel5_IRQn);

}

/** Configure pins as 
        * Analog 
        * Input 
        * Output
        * EVENT_OUT
        * EXTI
*/
static void MX_GPIO_Init(void)
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();

  /*Configure GPIO pins : name1_Pin name2_Pin */
  GPIO_InitStruct.Pin = name1_Pin|name2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */
//-----------------------------------------------------------------------------------------------------
//interupt timers


void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart) {
	timer_bluetooth++;
//	HAL_UART_Transmit_DMA(&huart1, Data, 140);
//	HAL_UART_Receive_DMA(&huart1, Received, 1); // start receive
}

uint8_t readRegister(uint16_t adress){
	uint8_t out[1];
	HAL_I2C_Mem_Read(&hi2c2, BME280_ADDRESS, adress, 1, &out, 1, 100);
	return out[0];
}
uint8_t writeRegister(uint16_t adress, uint8_t *pData){
	HAL_I2C_Mem_Write(&hi2c2, BME280_ADDRESS, adress, 1, &pData, 1, 100);
	return 1;
}

uint8_t bme280_callibration(){

//	//Reset
//	Settings=0xB6; //RESET
//	writeRegister(0xE0, Settings);

    //ctrl_meas		4B=  0100  1011   ->
	//Settings=0x27; //oversampling x1 + normal mode
	Settings=0xB7;	//oversampling x16 + normal mode
	writeRegister(0xF4, Settings);

	//Ctrl_hum
	Settings=0x02; //oversampling x2
	writeRegister(0xF2, Settings);

	//config
	Settings=0xB0; //tstandby 1s Filter 16, spi disable
	writeRegister(0xF5, Settings);


	//FIRST MEASURE
	//=========CALLIBRATION==========
	//cal temp
	calibration_dig_T1 = ((uint16_t)((readRegister(BME280_DIG_T1_MSB_REG) << 8) + readRegister(BME280_DIG_T1_LSB_REG)));
	calibration_dig_T2 = ((int16_t)((readRegister(BME280_DIG_T2_MSB_REG) << 8) + readRegister(BME280_DIG_T2_LSB_REG)));
	calibration_dig_T3 = ((int16_t)((readRegister(BME280_DIG_T3_MSB_REG) << 8) + readRegister(BME280_DIG_T3_LSB_REG)));
	//cal pressure
	calibration_dig_P1 = ((uint16_t)((readRegister(BME280_DIG_P1_MSB_REG) << 8) + readRegister(BME280_DIG_P1_LSB_REG)));
	calibration_dig_P2 = ((int16_t)((readRegister(BME280_DIG_P2_MSB_REG) << 8) + readRegister(BME280_DIG_P2_LSB_REG)));
	calibration_dig_P3 = ((int16_t)((readRegister(BME280_DIG_P3_MSB_REG) << 8) + readRegister(BME280_DIG_P3_LSB_REG)));
	calibration_dig_P4 = ((int16_t)((readRegister(BME280_DIG_P4_MSB_REG) << 8) + readRegister(BME280_DIG_P4_LSB_REG)));
	calibration_dig_P5 = ((int16_t)((readRegister(BME280_DIG_P5_MSB_REG) << 8) + readRegister(BME280_DIG_P5_LSB_REG)));
	calibration_dig_P6 = ((int16_t)((readRegister(BME280_DIG_P6_MSB_REG) << 8) + readRegister(BME280_DIG_P6_LSB_REG)));
	calibration_dig_P7 = ((int16_t)((readRegister(BME280_DIG_P7_MSB_REG) << 8) + readRegister(BME280_DIG_P7_LSB_REG)));
	calibration_dig_P8 = ((int16_t)((readRegister(BME280_DIG_P8_MSB_REG) << 8) + readRegister(BME280_DIG_P8_LSB_REG)));
	calibration_dig_P9 = ((int16_t)((readRegister(BME280_DIG_P9_MSB_REG) << 8) + readRegister(BME280_DIG_P9_LSB_REG)));
	//cal humidity
	calibration_dig_H1 = ((uint8_t)(readRegister(BME280_DIG_H1_REG)));
	calibration_dig_H2 = ((int16_t)((readRegister(BME280_DIG_H2_MSB_REG) << 8) + readRegister(BME280_DIG_H2_LSB_REG)));
	calibration_dig_H3 = ((uint8_t)(readRegister(BME280_DIG_H3_REG)));
	calibration_dig_H4 = ((int16_t)((readRegister(BME280_DIG_H4_MSB_REG) << 4) + (readRegister(BME280_DIG_H4_LSB_REG) & 0x0F)));
	calibration_dig_H5 = ((int16_t)((readRegister(BME280_DIG_H5_MSB_REG) << 4) + ((readRegister(BME280_DIG_H4_LSB_REG) >> 4) & 0x0F)));
	calibration_dig_H6 = ((uint8_t)readRegister(BME280_DIG_H6_REG));

	//=========CALLIBRATION END=======

	return 1;
}

uint8_t bme280_state(uint8_t state){
	if(state){
	Settings=0xB7;	//oversampling x16 + normal mode

	}
	else{
		Settings=0xB4;	//oversampling x16 + sleep mode
	}
	writeRegister(0xF4, Settings);
	return 1;
}
float bme280_getTemperature(){

	int32_t adc_T;
	int64_t tvar1, tvar2;
	float toutput;
	float Temperature_float;

	//=========TEMPERATURE=============
	adc_T = ((uint32_t)readRegister(BME280_TEMPERATURE_MSB_REG) << 12) | ((uint32_t)readRegister(BME280_TEMPERATURE_LSB_REG) << 4) | ((readRegister(BME280_TEMPERATURE_XLSB_REG) >> 4) & 0x0F);
	tvar1 = ((((adc_T>>3) - ((int32_t)calibration_dig_T1<<1))) * ((int32_t)calibration_dig_T2)) >> 11;
	tvar2 = (((((adc_T>>4) - ((int32_t)calibration_dig_T1)) * ((adc_T>>4) - ((int32_t)calibration_dig_T1))) >> 12) * ((int32_t)calibration_dig_T3)) >> 14;
	t_fine = tvar1 + tvar2;
	toutput = (t_fine * 5 + 128) >> 8;
	Temperature_float = toutput / 100;
	//=========TEMPERATURE END=========

	return Temperature_float;
}

uint16_t bme280_getHumidity(){

	int32_t adc_H;
	int32_t hvar1;
	float Humidity_float;
	uint32_t hum;

	//=========HUMIDITY =============
	adc_H = ((uint32_t)readRegister(BME280_HUMIDITY_MSB_REG) << 8) | ((uint32_t)readRegister(BME280_HUMIDITY_LSB_REG));
	hvar1 = (t_fine - ((int32_t)76800));
	hvar1 = (((((adc_H << 14) - (((int32_t)calibration_dig_H4) << 20) - (((int32_t)calibration_dig_H5) * hvar1)) + ((int32_t)16384)) >> 15) * (((((((hvar1 * ((int32_t)calibration_dig_H6)) >> 10) * (((hvar1 * ((int32_t)calibration_dig_H3)) >> 11) + ((int32_t)32768))) >> 10) + ((int32_t)2097152)) * ((int32_t)calibration_dig_H2) + 8192) >> 14));
	hvar1 = (hvar1 - (((((hvar1 >> 15) * (hvar1 >> 15)) >> 7) * ((int32_t)calibration_dig_H1)) >> 4));
	hvar1 = (hvar1 < 0 ? 0 : hvar1);
	hvar1 = (hvar1 > 419430400 ? 419430400 : hvar1);
	hum=((hvar1>>12));
	Humidity_float =(float)((hvar1>>12) >> 10);
	//=========HUMIDITY END==========

	return ((uint16_t) Humidity_float);
}

uint16_t bme280_getPressure(){

	int32_t adc_P;
	int64_t pvar1, pvar2, p_acc;
	float Pressure_float;

	//=========PRESSURE =============
	adc_P = ((uint32_t)readRegister(BME280_PRESSURE_MSB_REG) << 12) | ((uint32_t)readRegister(BME280_PRESSURE_LSB_REG) << 4) | ((readRegister(BME280_PRESSURE_XLSB_REG) >> 4) & 0x0F);
	pvar1 = ((int64_t)t_fine) - 128000;
	pvar2 = pvar1 * pvar1 * (int64_t)calibration_dig_P6;
	pvar2 = pvar2 + ((pvar1 * (int64_t)calibration_dig_P5)<<17);
	pvar2 = pvar2 + (((int64_t)calibration_dig_P4)<<35);
	pvar1 = ((pvar1 * pvar1 * (int64_t)calibration_dig_P3)>>8) + ((pvar1 * (int64_t)calibration_dig_P2)<<12);
	pvar1 = (((((int64_t)1)<<47)+pvar1))*((int64_t)calibration_dig_P1)>>33;
	if (pvar1 == 0)
	{
		return 0; // avoid exception caused by division by zero
	}
	p_acc = 1048576 - adc_P;
	p_acc = (((p_acc<<31) - pvar2)*3125)/pvar1;
	pvar1 = (((int64_t)calibration_dig_P9) * (p_acc>>13) * (p_acc>>13)) >> 25;
	pvar2 = (((int64_t)calibration_dig_P8) * p_acc) >> 19;
	p_acc = ((p_acc + pvar1 + pvar2) >> 8) + (((int64_t)calibration_dig_P7)<<4);
	p_acc = p_acc >> 8; // /256
	Pressure_float = ((float)p_acc)/100;
	//=========PRESSURE END==========

	return ((uint16_t) Pressure_float);
}

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void _Error_Handler(char * file, int line)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler_Debug */ 
}

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */

}

#endif

/**
  * @}
  */ 

/**
  * @}
*/ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
